@@ -62,7 +62,7 @@ void BuildDepCommand::set_argument_parser() {
             return true;
         });
     specs->set_complete_hook_func([&ctx](const char * arg) {
-        return match_specs(ctx, arg, false, true, true, false, ".*\\.(spec|src\\.rpm|nosrc\\.rpm)");
+        return ctx.match_specs(arg, false, true, true, false, ".*\\.(spec|src\\.rpm|nosrc\\.rpm)");
     });
     specs->set_nrepeats(ArgumentParser::PositionalArg::AT_LEAST_ONE);
     cmd.register_positional_arg(specs);
@@ -124,6 +124,7 @@ void BuildDepCommand::set_argument_parser() {
     create_allow_downgrade_options(*this);
 
     create_from_repo_option(*this, from_repos, true);
+    create_from_vendor_option(*this, from_vendors, true);
 
     create_store_option(*this);
     create_offline_option(*this);
@@ -279,6 +280,9 @@ bool BuildDepCommand::add_from_pkg(
     if (!from_repos.empty()) {
         pkg_query.filter_repo_id(from_repos, libdnf5::sack::QueryCmp::GLOB);
     }
+    if (!from_vendors.empty()) {
+        pkg_query.filter_vendor(from_vendors, libdnf5::sack::QueryCmp::GLOB);
+    }
     libdnf5::ResolveSpecSettings settings;
     settings.set_with_provides(false);
     settings.set_with_filenames(false);
@@ -295,6 +299,9 @@ bool BuildDepCommand::add_from_pkg(
     if (!from_repos.empty()) {
         source_pkgs.filter_repo_id(from_repos, libdnf5::sack::QueryCmp::GLOB);
     }
+    if (!from_vendors.empty()) {
+        source_pkgs.filter_vendor(from_vendors, libdnf5::sack::QueryCmp::GLOB);
+    }
     source_pkgs.filter_arch(std::vector<std::string>{"src", "nosrc"});
     source_pkgs.filter_name(source_names);
     if (source_pkgs.empty()) {
