@@ -106,9 +106,9 @@ void RepoqueryCommand::set_argument_parser() {
         });
     keys->set_complete_hook_func([this, &ctx](const char * arg) {
         if (this->installed_option->get_value()) {
-            return match_specs(ctx, arg, true, false, false, false);
+            return ctx.match_specs(arg, true, false, false, false);
         } else {
-            return match_specs(ctx, arg, false, true, true, false);
+            return ctx.match_specs(arg, false, true, true, false);
         }
     });
     cmd.register_positional_arg(keys);
@@ -842,16 +842,21 @@ void RepoqueryCommand::run() {
 
     if (srpm->get_value()) {
         libdnf5::rpm::PackageQuery srpms(ctx.get_base(), libdnf5::sack::ExcludeFlags::APPLY_EXCLUDES, true);
-        auto only_src_query = result_query;
+
+        libdnf5::rpm::PackageQuery only_src_query(ctx.get_base());
         only_src_query.filter_arch(std::vector<std::string>{"src", "nosrc"});
+
         for (const auto & pkg : result_query) {
-            if (!pkg.get_sourcerpm().empty()) {
-                auto tmp_q = only_src_query;
-                tmp_q.filter_name({pkg.get_source_name()});
-                tmp_q.filter_evr({pkg.get_evr()});
-                srpms |= tmp_q;
+            if (pkg.get_sourcerpm().empty()) {
+                continue;
             }
+
+            auto tmp_q = only_src_query;
+            tmp_q.filter_name({pkg.get_source_name()});
+            tmp_q.filter_evr({pkg.get_evr()});
+            srpms |= tmp_q;
         }
+
         result_query = srpms;
     }
 
