@@ -38,6 +38,7 @@
 #include "libdnf5/comps/comps_sack.hpp"
 #include "libdnf5/conf/config_parser.hpp"
 #include "libdnf5/conf/const.hpp"
+#include "libdnf5/plugin/utils.hpp"
 #include "libdnf5/utils/bgettext/bgettext-mark-domain.h"
 
 #include <atomic>
@@ -204,13 +205,7 @@ void Base::load_plugins() {
         return;
     }
 
-    const char * plugins_config_dir = std::getenv("LIBDNF_PLUGINS_CONFIG_DIR");
-    if (plugins_config_dir &&
-        p_impl->config.get_pluginconfpath_option().get_priority() < Option::Priority::COMMANDLINE) {
-        p_impl->plugins.load_plugins(plugins_config_dir);
-    } else {
-        p_impl->plugins.load_plugins(p_impl->config.get_pluginconfpath_option().get_value());
-    }
+    p_impl->plugins.load_plugins(plugin::get_config_dirs(*this));
 }
 
 void Base::add_plugin(
@@ -328,6 +323,18 @@ void Base::setup() {
     p_impl->plugins.post_base_setup();
 }
 
+bool Base::lock_system_repo(libdnf5::utils::LockAccess access, libdnf5::utils::LockBlocking blocking) {
+    return p_impl->lock_system_repo(access, blocking);
+}
+
+void Base::unlock_system_repo() {
+    p_impl->unlock_system_repo();
+}
+
+const libdnf5::utils::Locker * Base::get_system_repo_lock() const noexcept {
+    return p_impl->get_system_repo_lock();
+}
+
 bool Base::is_initialized() {
     return p_impl->pool.get() != nullptr;
 }
@@ -351,6 +358,11 @@ bool Base::are_repos_configured() const noexcept {
 ConfigMain & Base::get_config() {
     return p_impl->config;
 }
+
+const ConfigMain & Base::get_config() const {
+    return p_impl->config;
+}
+
 LogRouterWeakPtr Base::get_logger() {
     return {&p_impl->log_router, &p_impl->log_router_guard};
 }
@@ -430,4 +442,30 @@ libdnf5::system::State & Base::Impl::get_system_state() {
     return *system_state;
 }
 
+bool Base::Impl::lock_system_repo(libdnf5::utils::LockAccess access, libdnf5::utils::LockBlocking blocking) {
+    if (!system_repo_lock.has_value()) {
+        auto & installroot_option = config.get_installroot_option();
+        installroot_option.lock("Locked by Base::Impl::lock_system_repo()");
+        const auto & system_state_dir = config.get_system_state_dir_option().get_value();
+
+        const auto & relative_path =
+            std::filesystem::path{system_state_dir}.relative_path() / std::filesystem::path{SYSTEM_REPO_LOCK_FILENAME};
+        const auto & lock_file_path = installroot_option.get_value() / relative_path;
+
+        std::filesystem::create_directories(lock_file_path.parent_path());
+        system_repo_lock = libdnf5::utils::Locker{lock_file_path, true};
+    }
+    return system_repo_lock->lock(access, blocking);
+}
+
+void Base::Impl::unlock_system_repo() {
+    if (system_repo_lock.has_value()) {
+        system_repo_lock->unlock();
+    }
+}
+
+const libdnf5::utils::Locker * Base::Impl::get_system_repo_lock() const noexcept {
+    return system_repo_lock ? std::to_address(system_repo_lock) : nullptr;
+}
+
 }  // namespace libdnf5
