@@ -78,6 +78,18 @@ void construct_job(
         }
     }
 
+    // Automatically mark all protected packages as user installed.
+    // When a protected package is installed as a dependency it can block
+    // removal of the last package that depends on it (because the protected
+    // package cannot be removed, not even as an unused dependency).
+    // To prevent this and still correctly resolve dependencies of the protected
+    // packages mark them all as user installed.
+    if (protected_packages != nullptr) {
+        for (auto id : *protected_packages) {
+            job.push_back(SOLVER_SOLVABLE | SOLVER_USERINSTALLED, id);
+        }
+    }
+
     if (allow_erasing) {
         allow_uninstall_all_but_protected(pool, job, protected_packages, protected_kernel);
     }
